/***********************************************************************************
 * @Copyright (c)   :(C)2020, Qindao ieslab Co., Ltd
 * @FileName        :hc32_uart_driver.c
 * @Author          :
 * @Version         :V1.0
 * @Date            :2020-7-1
 * @Description :the function of the entity of system processor
 ************************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "sys_config.h"
#include "UWater_uart_driver.h"
#include "UWater_timer1uS_driver.h"
#include "UWater_ir_app.h"
#include "reset.h"

/* Private variables ---------------------------------------------------------*/
static UART_STRUCT uart_type[COM_MAX_NUM];
u8 ir_tx_end_rdy_flag;				//ir txԤ־
#if COM0_OPEN
static u8 s_uart0_rcv_buf[UART0_BUF_SIZE];  //0ջ
#endif

#if COM1_OPEN
static u8 s_uart1_rcv_buf[UART1_BUF_SIZE];  //1ջ
#endif

#if COM2_OPEN
static u8 s_uart2_rcv_buf[UART2_BUF_SIZE];  //2ջ
#endif

#if COM3_OPEN
static u8 s_uart3_rcv_buf[UART3_BUF_SIZE];  //3ջ
#endif

#if LPCOM0_OPEN
static u8 s_lpuart0_rcv_buf[LPUART0_BUF_SIZE];  //͹Ĵ0ջ
#endif

#if LPCOM1_OPEN
static u8 s_lpuart1_rcv_buf[LPUART1_BUF_SIZE];  //͹Ĵ1ջ
#endif

/* Private function prototypes -----------------------------------------------*/
static void UartRcvMachine(void);        //ڽǰ̨״̬
void UartSendMachine(void);
static void UartPortCfg(u8 com);         //IOжϿʼ
static void UartStartReceiveIT(u8 com);  //жϽ
static void UartStopReceiveIT(u8 com);   //ֹͣжϽ
static void UartIRQHandler(u8 com);      //жϴ
static void UartReceiveIT(u8 com);       //ڽжϴ
static void UartTransmitIT(u8 com);      //ڷжϴ

static void HAL_UartCfg(u8 com);
//void IR_Uart_38K_Config(void);
//void IR_Uart_38K_Enable(boolean_t enCmd);
/* Exported functions -------------------------------------------------------*/
/*************************************************
Function:u8 UartInit(u8 com,u8 baud,u8 databit,u8 stop,u8 check,u8 lpmode)
Description: ڳʼ
Input:  com ڣbaudʣdatabitλstopֹͣλcheckУλ;
lpmode5lpЧѡǷΪ͹ĴʹãUART_LPUART_DisableUART_LPUART_Enable
return: SUCCESSʼɹERROR ʼʧ
Others: MainSpaceһӿڣϵʼӿڡɹѡ񴮿12345lp
*************************************************/
u8 UartInit(u8 com, u8 baud, u8 databit, u8 stop, u8 check, u8 lpmode)
{
    switch (com)  //ѡҪõĴ
    {
#if COM0_OPEN
		case (COM0): 
		{
		    uart_type[com].Instance = M0P_UART0;
		} 
		break;
#endif
		
#if COM1_OPEN
		case (COM1): 
		{
		    uart_type[com].Instance = M0P_UART1;
		} 
		break;
#endif

#if COM2_OPEN
		case (COM2): 
		{
		    uart_type[com].Instance = M0P_UART2;
		} 
		break;
#endif

#if COM3_OPEN
		case (COM3): 
		{
		    uart_type[com].Instance = M0P_UART3;
		} 
		break;
#endif

#if LPCOM0_OPEN
		case (LPCOM0): 
		{
		    uart_type[com].LP_Instance = M0P_LPUART0;
		} 
		break;
#endif

#if LPCOM1_OPEN
		case (LPCOM1): 
		{
		    uart_type[com].LP_Instance = M0P_LPUART1;
		} 
		break;
#endif
        default: 
		{
			return ERROR;
		}
    }
    uart_type[com].uart_status = TASK_IDLE;
    switch (baud)  //ò
    {
		case (BAUD1200): 
		{
			uart_type[com].para.BaudRate = 1200;
		} 
		break;

		case (BAUD2400): 
		{
		    uart_type[com].para.BaudRate = 2400;
		} 
		break;

		case (BAUD4800): 
		{
		    uart_type[com].para.BaudRate = 4800;
		} 
		break;

		case (BAUD9600): 
		{
		    uart_type[com].para.BaudRate = 9600;
		} 
		break;

		case (BAUD19200): 
		{
		    uart_type[com].para.BaudRate = 19200;
		} 
		break;

		case (BAUD38400): 
		{
		    uart_type[com].para.BaudRate = 38400;
		} 
		break;

		case (BAUD57600): 
		{
		    uart_type[com].para.BaudRate = 57600;
		} 
		break;

		case (BAUD115200): 
		{
		    uart_type[com].para.BaudRate = 115200;
		} 
		break;

		default: 
		{
			return ERROR;
		}
    }

    switch (databit)  //ݳ
    {
		case (DATA_BIT_8): 
		{
			uart_type[com].para.WordLength = UART_WORDLEN_8B;
		} 
		break;
		
		default: 
		{
			return ERROR;
		}
    }

    switch (stop)  //ֹͣλ
    {
		case (STOPBIT_1): 
		{
			uart_type[com].para.StopBits = UART_STOPBIT_1;
		} 
		break;

		case (STOPBIT_2): 
		{
		    uart_type[com].para.StopBits = UART_STOPBIT_2;
		} 
		break;

		case (STOPBIT_1_5): 
		{
		    uart_type[com].para.StopBits = UART_STOPBIT_1_5;
		} 
		break;

		default: 
		{
			return ERROR;
		}
    }

    switch (check)  //Уλ
    {
		case (UART_NONE_PARITY): 
		{
			uart_type[com].para.Parity = UART_NONE_PARITY;
			uart_type[com].para.UARTMode = UartMskMode1;
		} 
		break;

		case (UART_ODD_PARITY): 
		{
		    uart_type[com].para.Parity = UART_ODD_PARITY;
		    uart_type[com].para.UARTMode = UartMskMode3;
		} 
		break;

		case (UART_EVEN_PARITY): 
		{
		    uart_type[com].para.Parity = UART_EVEN_PARITY;
		    uart_type[com].para.UARTMode = UartMskMode3;
		} 
		break;

		default: 
		{
			return ERROR;
		}
    }          
    uart_type[com].para.OverSampling = UartMsk8Or16Div;//UartMsk16Or32Div;  // 16λ
    
    if((LPCOM0 == com)||(LPCOM1 == com))
	{
		uart_type[com].para.LPModeFunction = lpmode;  //ѡ񴮿Ƿʹõ͹Ĺ
    }
    UartPortCfg(com);  //ùܽżжȼ
    HAL_UartCfg(com);
    UartStartReceiveIT(com);  //ж
    return SUCCESS;
}


void IrTxDealMachine(void)
{
	if(TRUE == ir_tx_end_rdy_flag)
	{
		if (0 == Check100msTimer(TIMER_100MS_IR_TX_38K_END)) //TX BUF 
		{
			ir_tx_end_rdy_flag = FALSE;
			IR_38KCarrier_Close();
		}
	} 
}

/*************************************************
Function: void UartMachineDriver (void)
Description:ǰ̨
Input:  None
Return: None
Others: MainSpaceڶӿڣӿڡ
*************************************************/
void UartMachineDriver(void)
{    
    UartRcvMachine();
	UartSendMachine();
	IrTxDealMachine();
}
/*************************************************
Function: u8 UartIfIdle (u8 com)
Description: Ƿ
Input:  com˿ں
return: TASK_IDLE:УTASK_BUSY:æµ
Others: MainSpaceڶӿڣӿڡ
*************************************************/
u8 UartIfIdle(u8 com)
{
	if(com < COM_MAX_NUM)
	{
		return uart_type[com].uart_status;
	}
	else
	{
		return TASK_IDLE;
	}	  
}

/*************************************************
Function:void UartSend (u8 com, u16 len, u8 *data,u8 *msg)
Description: ڷ
Input:  com˿ںţlen:ݳȣ*dataҪ͵ݣ*msgϢ
return: TASK_IDLE:УTASK_BUSY:æµ
Others: MainSpaceڶӿڣӿڡ
*************************************************/
void UartSend(u8 com, u16 len, u8 *data, u8 *out_msg)
{
	if(com < COM_MAX_NUM)
	{
        if((len > 0) && (len < UP_BUF_SIZE*2))//Է͵ݳȽ
        {
            uart_type[com].send_point = data;
            uart_type[com].send_len = len;
            uart_type[com].send_status_point = out_msg;
            *uart_type[com].send_status_point = TASK_BUSY;
            uart_type[com].uart_status = TASK_BUSY;	
			uart_type[com].start_send_flag = TRUE;	
			uart_type[com].send_over_time = uart_type[com].send_len/10;  //100msٷ10ֽ
			if(uart_type[com].send_over_time < 30)
			{
				uart_type[com].send_over_time = 30;  //3
			}				
			if(LPCOM0 == com)//
			{
				IR_38KCarrier_Open();
				ir_tx_end_rdy_flag = FALSE;				
				LPUart_DisableIrq(uart_type[com].LP_Instance,LPUartRxIrq);   //ʱرսգụ
				LPUart_ClrStatus(uart_type[com].LP_Instance,LPUartTC);      ///<巢ж				
				LPUart_SendDataIt(uart_type[com].LP_Instance, *uart_type[com].send_point);
				uart_type[com].send_point++;
				uart_type[com].send_len--;            
				LPUart_EnableIrq(uart_type[com].LP_Instance, LPUartTxIrq);   ///<ж
			}
			else if(LPCOM1 == com)//NB
			{					
				LPUart_SendDataIt(uart_type[com].LP_Instance, *uart_type[com].send_point);
				uart_type[com].send_point++;
				uart_type[com].send_len--;            
				LPUart_EnableIrq(uart_type[com].LP_Instance, LPUartTxIrq);   ///<ж
			}
			else
			{
				Uart_SendDataIt(uart_type[com].Instance, *uart_type[com].send_point); 		  //
				uart_type[com].send_point++;
				uart_type[com].send_len--;            
				Uart_EnableIrq(uart_type[com].Instance, UartTxIrq);  	   ///<ж	
            }
		}
	}
}
/*************************************************
Function: u8 UartRcvFinish (u8 com)
Description: ڽ
Input:  com:˿ں
Return: None
Others: MainSpaceڶӿڣӿڡ
*************************************************/
u8 UartRcvFinish(u8 com)
{
	if(com < COM_MAX_NUM)
	{
		if (TRUE == uart_type[com].rcv_msg) 
		{
			return TRUE;
		} 
		else 
		{
		    return FALSE;
		}
    }
	else
	{
        return FALSE;
    }
}

/*************************************************
Function: void UartClearMsg (u8 com)
Description: ڽ¼Msg,ʹܽ
Input:  com ˿ں
Return: None
Others: MainSpaceڶӿڣӿڡ
*************************************************/
void UartClearMsg(u8 com)
{
    if(com < COM_MAX_NUM)
	{
        uart_type[com].rcv_msg = FALSE;
        uart_type[com].rcv_len = 0;
        UartStartReceiveIT(com);  //ж
    } 
}
/*************************************************
Function: void UartRcv (u8 com ,u16 *out_len , u8 **out_data)
Description: ڽ
Input:  com˿ںţout_lenݳȣout_data:ָ
Return: None
Others: MainSpaceڶӿڣӿڡ
*************************************************/
void UartRcv(u8 com, u16 *out_len, u8 **out_data)
{
	if(com < COM_MAX_NUM)
	{
		*out_len = uart_type[com].rcv_len;
		switch (com)
		{
#if COM0_OPEN
			case (COM0): 
			{
			    *out_data = s_uart0_rcv_buf;
			} 
			break;
#endif
			
#if COM1_OPEN
			case (COM1): 
			{
			    *out_data = s_uart1_rcv_buf;
			} 
			break;			
#endif

#if COM2_OPEN
			case (COM2): 
			{			  
			    *out_data = s_uart2_rcv_buf;
			} 
			break;
#endif

#if COM3_OPEN
			case (COM3): 
			{			    
			    *out_data = s_uart3_rcv_buf;
			} 
			break;
#endif

#if LPCOM0_OPEN
			case (LPCOM0): 
			{
			    *out_data = s_lpuart0_rcv_buf;
			} 
			break;
#endif

#if LPCOM1_OPEN
			case (LPCOM1): 
			{
			    *out_data = s_lpuart1_rcv_buf;
			} 
			break;
#endif
			default:
			break;
		}
	}
}
/*************************************************
Function: u8 UartIfSleep(u8 com)
Description: Ƿ
Input:  None
Return: TASK_IDLE:TASK_BUSY
Others: MainSpaceӿڣǰӿڡ
*************************************************/
u8 UartIfSleep(u8 com)
{
	if(com < COM_MAX_NUM)
	{
		return uart_type[com].uart_status;
	}
	else
	{
		return TASK_IDLE;
    } 	
}
/*************************************************
Function: void UartPreSleep(u8 com)
Description: ǰ
Input:  com:˿ں
Return: None
Others: MainSpaceӿڣǰӿڡ
*************************************************/
void UartPreSleep(u8 com)
{
    switch (com) 
	{
#if COM0_OPEN
		case COM0: 
		{
		    uart_type[COM0].Instance->SCON = 0;    //رй
		    M0P_SYSCTRL->PERI_CLKEN0_f.UART0 = 0;  //رմʱ
		} 
		break;
#endif
		
#if COM1_OPEN
		case COM1: 
		{
		    uart_type[COM1].Instance->SCON = 0;    //رй
		    M0P_SYSCTRL->PERI_CLKEN0_f.UART1 = 0;  //رմʱ
		} 
		break;
#endif

#if COM2_OPEN
		case COM2: 
		{
		    uart_type[COM2].Instance->SCON = 0;    //رй
		    M0P_SYSCTRL->PERI_CLKEN1_f.UART2 = 0;  //رմʱ
		} 
		break;
#endif

#if COM3_OPEN

		case COM3: 
		{
		    uart_type[COM3].Instance->SCON = 0;    //رй
		    M0P_SYSCTRL->PERI_CLKEN1_f.UART3 = 0;  //رմʱ
		} 
		break;
#endif

#if LPCOM0_OPEN
		case LPCOM0:
		{
	#if LPCOM0_ENABLESLEEPMODE
		    uart_type[LPCOM1].LP_Instance->SCON_f.RCIE = 1;
		    uart_type[LPCOM1].LP_Instance->SCON_f.REN = 1;
	#else
		    uart_type[LPCOM0].LP_Instance->SCON = 0;  //رй
		    M0P_SYSCTRL->PERI_CLKEN0_f.LPUART0 = 0;   //رմʱ
	#endif
		} break;
#endif

#if LPCOM1_OPEN
		case LPCOM1: 
		{
	#if LPCOM1_ENABLESLEEPMODE
		    uart_type[LPCOM1].LP_Instance->SCON_f.RCIE = 1;//жϴ
		    uart_type[LPCOM1].LP_Instance->SCON_f.REN = 1; //жϴ

	#else
		    uart_type[LPCOM1].LP_Instance->SCON = 0;  //رй
		    M0P_SYSCTRL->PERI_CLKEN0_f.LPUART1 = 0;   //رմʱ
	#endif
		}
		break;
#endif

        default:
        break;
    }
}
/*************************************************
Function:void UartWakeSleep(u8 com,u8 baud,u8 databit,u8 stop,u8 check,lpmode)
Description: ڻ
Input:  Input:com
ڣbaudʣdatabitλstopֹͣλcheckУλ; return: None
Others: MainSpaceӿڣѺӿڡ
*************************************************/
void UartWakeSleep(u8 com, u8 baud, u8 databit, u8 stop, u8 check, u8 lpmode)
{
    UartInit(com, baud, databit, stop, check, lpmode);
}

/*************************************************
Function:void USART0_IRQHandler(void)
Description: 0жں
Input:  None
Return: None
Others: None
*************************************************/
#if COM0_OPEN
void Uart0_IRQHandler(void) { UartIRQHandler(COM0); }
#endif

/*************************************************
Function:void USART1_IRQHandler(void)
Description: 1жں
Input:  None
Return: None
Others: None
*************************************************/
#if COM1_OPEN
void Uart1_IRQHandler(void) { UartIRQHandler(COM1); }
#endif

/*************************************************
Function:void USART2_IRQHandler(void)
Description: 2жں
Input:  None
Return: None
Others: None
*************************************************/
#if COM2_OPEN
void Uart2_IRQHandler(void) { UartIRQHandler(COM2); }
#endif

/*************************************************
Function:void UART3_IRQHandler(void)
Description: 3жں
Input:  None
Return: None
Others: None
*************************************************/
#if COM3_OPEN
void Uart3_IRQHandler(void) { UartIRQHandler(COM3); }
#endif

/*************************************************
Function:void UART5_IRQHandler(void)
Description: 5жں
Input:  None
Return: None
Others: None
*************************************************/
#if LPCOM0_OPEN
void LpUart0_IRQHandler(void) { UartIRQHandler(LPCOM0); }
#endif

/*************************************************
Function:void LPUART1_IRQHandler(void)
Description: ͹Ĵжں
Input:  None
Return: None
Others: None
*************************************************/
#if LPCOM1_OPEN
void LpUart1_IRQHandler(void) { UartIRQHandler(LPCOM1); }
#endif

/*************************************************
Function:void UartRcvMachine(void)
Description: ڽǰ̨״̬
Input:  None
return: None
Others: ڲӿ
*************************************************/
static void UartRcvMachine(void)
{
#if COM0_OPEN
    switch (uart_type[COM0].rcv_state) 
	{
		case (UART_RCV_OVERTIME_S0): 
		{
		    if (1 == uart_type[COM0].rcv_flag) 
			{
				uart_type[COM0].rcv_flag = 0;
				Set5msTimer(TIMER_5MS_UART0, FRAMEOVERTIME);
				uart_type[COM0].rcv_state = UART_RCV_OVERTIME_S1;
		    } 
		} 
		break;

		case (UART_RCV_OVERTIME_S1): 
		{
			if (1 == uart_type[COM0].rcv_flag) 
			{
				uart_type[COM0].rcv_flag = 0;
				Set5msTimer(TIMER_5MS_UART0, FRAMEOVERTIME);
				uart_type[COM0].rcv_state = UART_RCV_OVERTIME_S1;
			}
			else if ((0 == uart_type[COM0].rcv_flag) &&(0 == Check5msTimer(TIMER_5MS_UART0))) 
			{
				uart_type[COM0].rcv_msg = TRUE;				
				uart_type[COM0].rcv_state = UART_RCV_OVERTIME_S0;
				UartStopReceiveIT(COM0);
		    } 
		} 
		break;

		default: 
		{
			uart_type[COM0].rcv_state = UART_RCV_OVERTIME_S0;
		} 
		break;
  }
#endif	
	
#if COM1_OPEN
	switch (uart_type[COM1].rcv_state) 
	{
		case (UART_RCV_OVERTIME_S0): 
		{
			if (1 == uart_type[COM1].rcv_flag) 
			{
				uart_type[COM1].rcv_flag = 0;
				Set5msTimer(TIMER_5MS_UART1, FRAMEOVERTIME);
				uart_type[COM1].rcv_state = UART_RCV_OVERTIME_S1;
		    }
		} 
		break;

		case (UART_RCV_OVERTIME_S1):
		{
			if (1 == uart_type[COM1].rcv_flag) 
			{
				uart_type[COM1].rcv_flag = 0;
				Set5msTimer(TIMER_5MS_UART1, FRAMEOVERTIME);
				uart_type[COM1].rcv_state = UART_RCV_OVERTIME_S1;
			} 
			else if ((0 == uart_type[COM1].rcv_flag) && (0 == Check5msTimer(TIMER_5MS_UART1))) 
			{
				uart_type[COM1].rcv_msg = TRUE;				
				uart_type[COM1].rcv_state = UART_RCV_OVERTIME_S0;
				UartStopReceiveIT(COM1);
			} 
		} 
		break;

		default: 
		{
			uart_type[COM1].rcv_state = UART_RCV_OVERTIME_S0;
		} 
		break;
	}
#endif

#if COM2_OPEN
	switch (uart_type[COM2].rcv_state) 
	{
		case (UART_RCV_OVERTIME_S0): 
		{
		    if (1 == uart_type[COM2].rcv_flag) 
			{
				uart_type[COM2].rcv_flag = 0;
				Set5msTimer(TIMER_5MS_UART2, FRAMEOVERTIME);
				uart_type[COM2].rcv_state = UART_RCV_OVERTIME_S1;
			}
		} 
		break;

		case (UART_RCV_OVERTIME_S1):
		{
			if (1 == uart_type[COM2].rcv_flag) 
			{
				uart_type[COM2].rcv_flag = 0;
				Set5msTimer(TIMER_5MS_UART2, FRAMEOVERTIME);
				uart_type[COM2].rcv_state = UART_RCV_OVERTIME_S1;
			}
			else if ((0 == uart_type[COM2].rcv_flag) && (0 == Check5msTimer(TIMER_5MS_UART2))) 
			{
				uart_type[COM2].rcv_msg = TRUE;				
				uart_type[COM2].rcv_state = UART_RCV_OVERTIME_S0;
				UartStopReceiveIT(COM2);
			} 
		} 
		break;

		default: 
		{
		    uart_type[COM2].rcv_state = UART_RCV_OVERTIME_S0;
		} 
		break;
	}
#endif

#if COM3_OPEN
	switch (uart_type[COM3].rcv_state) 
	{
		case (UART_RCV_OVERTIME_S0): 
		{
		    if (1 == uart_type[COM3].rcv_flag) 
			{
				uart_type[COM3].rcv_flag = 0;
				Set5msTimer(TIMER_5MS_UART3, FRAMEOVERTIME);
				uart_type[COM3].rcv_state = UART_RCV_OVERTIME_S1;
			}
		} 
		break;

		case (UART_RCV_OVERTIME_S1): 
		{
			if (1 == uart_type[COM3].rcv_flag) 
			{
				uart_type[COM3].rcv_flag = 0;
				Set5msTimer(TIMER_5MS_UART3, FRAMEOVERTIME);
				uart_type[COM3].rcv_state = UART_RCV_OVERTIME_S1;
			}
			else if ((0 == uart_type[COM3].rcv_flag) && (0 == Check5msTimer(TIMER_5MS_UART3))) 
			{
				uart_type[COM3].rcv_msg = TRUE;				
				uart_type[COM3].rcv_state = UART_RCV_OVERTIME_S0;
				UartStopReceiveIT(COM3);
			} 			
		} 
		break;

		default: 
		{
		    uart_type[COM3].rcv_state = UART_RCV_OVERTIME_S0;
		} 
		break;
    }
#endif

#if LPCOM0_OPEN
	switch (uart_type[LPCOM0].rcv_state) 
	{
		case (UART_RCV_OVERTIME_S0): 
		{
			if (1 == uart_type[LPCOM0].rcv_flag) 
			{
				uart_type[LPCOM0].rcv_flag = 0;
				Set5msTimer(TIMER_5MS_LPUART0, FRAMEOVERTIME);
				uart_type[LPCOM0].rcv_state = UART_RCV_OVERTIME_S1;
		    }
		} 
		break;

		case (UART_RCV_OVERTIME_S1):
		{
			if (1 == uart_type[LPCOM0].rcv_flag) 
			{
				uart_type[LPCOM0].rcv_flag = 0;
				Set5msTimer(TIMER_5MS_LPUART0, FRAMEOVERTIME);
				uart_type[LPCOM0].rcv_state = UART_RCV_OVERTIME_S1;
			}
			else if ((0 == uart_type[LPCOM0].rcv_flag) && (0 == Check5msTimer(TIMER_5MS_LPUART0))) 
			{
				uart_type[LPCOM0].rcv_msg = TRUE;
				uart_type[LPCOM0].rcv_state = UART_RCV_OVERTIME_S0;
				UartStopReceiveIT(LPCOM0);
			} 
		} 
		break;

		default: 
		{
		    uart_type[LPCOM0].rcv_state = UART_RCV_OVERTIME_S0;
		} 
		break;
  }
#endif

#if LPCOM1_OPEN
	#if MODULE_BLE_USED
	switch (uart_type[LPCOM1].rcv_state) 
	{
		case (UART_RCV_OVERTIME_S0): 
		{
			if (1 == BleRcvFrameDataFlag())//if (1 == uart_type[LPCOM1].rcv_flag) 
			{
				BleClrRcvFrameDataFlag();//uart_type[LPCOM1].rcv_flag = 0;
				Set5msTimer(TIMER_5MS_LPUART1, FRAMEOVERTIME);
				uart_type[LPCOM1].rcv_state = UART_RCV_OVERTIME_S1;
			}
		}
		break;

		case (UART_RCV_OVERTIME_S1): 
		{
			if (1 == BleRcvFrameDataFlag())//if (1 == uart_type[LPCOM1].rcv_flag) 
			{
				BleClrRcvFrameDataFlag();//uart_type[LPCOM1].rcv_flag = 0;
				Set5msTimer(TIMER_5MS_LPUART1, FRAMEOVERTIME);
				uart_type[LPCOM1].rcv_state = UART_RCV_OVERTIME_S1;
			}
			else if ((0 == BleRcvFrameDataFlag()) && (0 == Check5msTimer(TIMER_5MS_LPUART1))) //else if ((0 == uart_type[LPCOM1].rcv_flag) && (0 == Check5msTimer(TIMER_5MS_LPUART1))) 
			{
				uart_type[LPCOM1].rcv_msg = TRUE;
				uart_type[LPCOM1].rcv_state = UART_RCV_OVERTIME_S0;
				UartStopReceiveIT(LPCOM1);
			} 
		} 
		break;

		default: 
		{
		    uart_type[LPCOM1].rcv_state = UART_RCV_OVERTIME_S0;
		} 
		break;
    }
	#else
	switch (uart_type[LPCOM1].rcv_state) 
	{
		case (UART_RCV_OVERTIME_S0): 
		{
			if (1 == uart_type[LPCOM1].rcv_flag) 
			{
				uart_type[LPCOM1].rcv_flag = 0;
				Set5msTimer(TIMER_5MS_LPUART1, FRAMEOVERTIME);
				uart_type[LPCOM1].rcv_state = UART_RCV_OVERTIME_S1;
			}
		}
		break;

		case (UART_RCV_OVERTIME_S1): 
		{
			if (1 == uart_type[LPCOM1].rcv_flag) 
			{
				uart_type[LPCOM1].rcv_flag = 0;
				Set5msTimer(TIMER_5MS_LPUART1, FRAMEOVERTIME);
				uart_type[LPCOM1].rcv_state = UART_RCV_OVERTIME_S1;
			}
			else if ((0 == uart_type[LPCOM1].rcv_flag) && (0 == Check5msTimer(TIMER_5MS_LPUART1))) 
			{
				uart_type[LPCOM1].rcv_msg = TRUE;
				uart_type[LPCOM1].rcv_state = UART_RCV_OVERTIME_S0;
				UartStopReceiveIT(LPCOM1);
			} 
		} 
		break;

		default: 
		{
		    uart_type[LPCOM1].rcv_state = UART_RCV_OVERTIME_S0;
		} 
		break;
    }
    #endif
#endif
}
//дڽտ
u8 UartRcvMachineIfIdle(u8 com)
{
	switch(com)
	{
#if COM0_OPEN
		case(COM0):
		{
			if(UART_RCV_OVERTIME_S0 == uart_type[COM0].rcv_state)
			{
				return TASK_IDLE;
			}
			else
			{
				return TASK_BUSY;
			}
		}
//		break;
#endif
	
#if COM1_OPEN
		case(COM1):
		{
			if(UART_RCV_OVERTIME_S0 == uart_type[COM1].rcv_state)
			{
				return TASK_IDLE;
			}
			else
			{
				return TASK_BUSY;
			}
		}
//		break;
#endif
		
#if COM2_OPEN
		case(COM2):
		{
			if(UART_RCV_OVERTIME_S0 == uart_type[COM2].rcv_state)
			{
				return TASK_IDLE;
			}
			else
			{
				return TASK_BUSY;
			}
		}
//		break;
#endif		
		
#if COM3_OPEN
		case(COM3):
		{
			if(UART_RCV_OVERTIME_S0 == uart_type[COM3].rcv_state)
			{
				return TASK_IDLE;
			}
			else
			{
				return TASK_BUSY;
			}
		}
//		break;
#endif			
		
#if LPCOM0_OPEN
		case(LPCOM0):
		{
			if(UART_RCV_OVERTIME_S0 == uart_type[LPCOM0].rcv_state)
			{
				return TASK_IDLE;
			}
			else
			{
				return TASK_BUSY;
			}
		}
//		break;
#endif		

#if LPCOM1_OPEN
		case(LPCOM1):
		{
			if(UART_RCV_OVERTIME_S0 == uart_type[LPCOM1].rcv_state)
			{
				return TASK_IDLE;
			}
			else
			{
				return TASK_BUSY;
			}
		}
//		break;
#endif		
		default:
		break;
	}
	return TASK_IDLE;
}
/*************************************************
Function:void UartSendMachine(void)
Description: ڷǰ̨״̬
Input:  None
return: None
Others: ڲӿ
*************************************************/
void UartSendMachine(void)
{
#if COM0_OPEN
	if(TASK_IDLE == uart_type[COM0].uart_status)
	{
		Set100msTimer(TIMER_100MS_UART0_BUSY,UART_BUSY_OVERTIME);
	}
	else if(0 == Check100msTimer(TIMER_100MS_UART0_BUSY))
	{
		uart_type[COM0].uart_status = TASK_IDLE;//ʱǿƸλ
		Reset_RstPeripheral0(ResetMskUart0);
		uart_type[COM0].finish_send_flag = FALSE;				
		UartInit(COM0,BAUD2400,DATA_BIT_8,STOPBIT_1,UART_NONE_PARITY,UART_LPUART_Disable);
		uart_type[COM0].send_state = UART_SEND_OVERTIME_S0;		
	}
    switch (uart_type[COM0].send_state) 
	{
		case (UART_SEND_OVERTIME_S0): 
		{
			if (TRUE == uart_type[COM0].start_send_flag) 
			{				
				uart_type[COM0].start_send_flag = FALSE;
				Set100msTimer(TIMER_100MS_UART0_SEND, uart_type[COM0].send_over_time);
				uart_type[COM0].send_state = UART_SEND_OVERTIME_S1;
		    }
		} 
		break;
		case (UART_SEND_OVERTIME_S1):
		{
			if (TRUE == uart_type[COM0].finish_send_flag) 
			{
				uart_type[COM0].finish_send_flag = FALSE;
				uart_type[COM0].send_state = UART_SEND_OVERTIME_S0;
			} 
			else if (0 == Check100msTimer(TIMER_100MS_UART0_SEND)) 
			{
				uart_type[COM0].uart_status = TASK_IDLE;//ͳʱǿƸλ
				Reset_RstPeripheral0(ResetMskUart0);				
				uart_type[COM0].finish_send_flag = FALSE;
				UartInit(COM0,BAUD2400,DATA_BIT_8,STOPBIT_1,UART_NONE_PARITY,UART_LPUART_Disable);
				uart_type[COM0].send_state = UART_SEND_OVERTIME_S0;
			} 
		} 
		break;
		default: 
		{
			uart_type[COM0].send_state = UART_SEND_OVERTIME_S0;
		} 
		break;
	}
#endif	
#if COM1_OPEN
	if(TASK_IDLE == uart_type[COM1].uart_status)
	{
		Set100msTimer(TIMER_100MS_UART1_BUSY,UART_BUSY_OVERTIME);
	}
	else if(0 == Check100msTimer(TIMER_100MS_UART1_BUSY))
	{
		uart_type[COM1].uart_status = TASK_IDLE;//ʱǿƸλ
		Reset_RstPeripheral0(ResetMskUart1);
		uart_type[COM1].finish_send_flag = FALSE;				
		UartInit(NB_UART_COM, BAUD9600, DATA_BIT_8, STOPBIT_1, UART_NONE_PARITY,UART_LPUART_Disable);	
		uart_type[COM1].send_state = UART_SEND_OVERTIME_S0;		
	}
	switch (uart_type[COM1].send_state) 
	{
		case (UART_SEND_OVERTIME_S0): 
		{
			if (TRUE == uart_type[COM1].start_send_flag) 
			{				
				uart_type[COM1].start_send_flag = FALSE;
				Set100msTimer(TIMER_100MS_UART1_SEND, uart_type[COM1].send_over_time);
				uart_type[COM1].send_state = UART_SEND_OVERTIME_S1;
		    }
		} 
		break;
		case (UART_SEND_OVERTIME_S1):
		{
			if (TRUE == uart_type[COM1].finish_send_flag) 
			{
				uart_type[COM1].finish_send_flag = FALSE;
				uart_type[COM1].send_state = UART_SEND_OVERTIME_S0;
			} 
			else if (0 == Check100msTimer(TIMER_100MS_UART1_SEND)) 
			{
				uart_type[COM1].uart_status = TASK_IDLE;//ͳʱǿƸλ
				Reset_RstPeripheral0(ResetMskUart1);
				uart_type[COM1].finish_send_flag = FALSE;				
				UartInit(NB_UART_COM, BAUD9600, DATA_BIT_8, STOPBIT_1, UART_NONE_PARITY,UART_LPUART_Disable);	
				uart_type[COM1].send_state = UART_SEND_OVERTIME_S0;
			} 
		} 
		break;
		default: 
		{
			uart_type[COM1].send_state = UART_SEND_OVERTIME_S0;
		} 
		break;
	}
#endif
#if COM2_OPEN
	if(TASK_IDLE == uart_type[COM2].uart_status)
	{
		Set100msTimer(TIMER_100MS_UART2_BUSY,UART_BUSY_OVERTIME);
	}
	else if(0 == Check100msTimer(TIMER_100MS_UART2_BUSY))
	{
		uart_type[COM2].uart_status = TASK_IDLE;//ʱǿƸλ
		Reset_RstPeripheral1(ResetMskUart2);
		uart_type[COM2].finish_send_flag = FALSE;				
		UartInit(COM2,BAUD2400,DATA_BIT_8,STOPBIT_1,UART_NONE_PARITY,UART_LPUART_Disable);
		uart_type[COM2].send_state = UART_SEND_OVERTIME_S0;		
	}
	switch (uart_type[COM2].send_state) 
	{
		case (UART_SEND_OVERTIME_S0): 
		{
			if (TRUE == uart_type[COM2].start_send_flag) 
			{				
				uart_type[COM2].start_send_flag = FALSE;
				Set100msTimer(TIMER_100MS_UART2_SEND, uart_type[COM2].send_over_time);
				uart_type[COM2].send_state = UART_SEND_OVERTIME_S1;
		    }
		} 
		break;
		case (UART_SEND_OVERTIME_S1):
		{
			if (TRUE == uart_type[COM2].finish_send_flag) 
			{
				uart_type[COM2].finish_send_flag = FALSE;
				uart_type[COM2].send_state = UART_SEND_OVERTIME_S0;
			} 
			else if (0 == Check100msTimer(TIMER_100MS_UART2_SEND)) 
			{
				uart_type[COM2].uart_status = TASK_IDLE;//ͳʱǿƸλ
				Reset_RstPeripheral1(ResetMskUart2);
				uart_type[COM2].finish_send_flag = FALSE;
				UartInit(COM2,BAUD2400,DATA_BIT_8,STOPBIT_1,UART_NONE_PARITY,UART_LPUART_Disable);
				uart_type[COM2].send_state = UART_SEND_OVERTIME_S0;
			} 
		} 
		break;
		default: 
		{
			uart_type[COM2].send_state = UART_SEND_OVERTIME_S0;
		} 
		break;
	}
#endif
#if COM3_OPEN
	if(TASK_IDLE == uart_type[COM3].uart_status)
	{
		Set100msTimer(TIMER_100MS_UART3_BUSY,UART_BUSY_OVERTIME);
	}
	else if(0 == Check100msTimer(TIMER_100MS_UART3_BUSY))
	{
		uart_type[COM3].uart_status = TASK_IDLE;//ʱǿƸλ
		Reset_RstPeripheral1(ResetMskUart3);
		uart_type[COM3].finish_send_flag = FALSE;				
		UartInit(COM3,BAUD2400,DATA_BIT_8,STOPBIT_1,UART_NONE_PARITY,UART_LPUART_Disable);
		uart_type[COM3].send_state = UART_SEND_OVERTIME_S0;		
	}
	switch (uart_type[COM3].send_state) 
	{
		case (UART_SEND_OVERTIME_S0): 
		{
			if (TRUE == uart_type[COM3].start_send_flag) 
			{				
				uart_type[COM3].start_send_flag = FALSE;
				Set100msTimer(TIMER_100MS_UART3_SEND, uart_type[COM3].send_over_time);
				uart_type[COM3].send_state = UART_SEND_OVERTIME_S1;
		    }
		} 
		break;
		case (UART_SEND_OVERTIME_S1):
		{
			if (TRUE == uart_type[COM3].finish_send_flag) 
			{
				uart_type[COM3].finish_send_flag = FALSE;
				uart_type[COM3].send_state = UART_SEND_OVERTIME_S0;
			} 
			else if (0 == Check100msTimer(TIMER_100MS_UART3_SEND)) 
			{
				uart_type[COM3].uart_status = TASK_IDLE;//ͳʱǿƸλ
				Reset_RstPeripheral1(ResetMskUart3);
				uart_type[COM3].finish_send_flag = FALSE;
				UartInit(COM3,BAUD2400,DATA_BIT_8,STOPBIT_1,UART_NONE_PARITY,UART_LPUART_Disable);
				uart_type[COM3].send_state = UART_SEND_OVERTIME_S0;
			} 
		} 
		break;
		default: 
		{
			uart_type[COM3].send_state = UART_SEND_OVERTIME_S0;
		} 
		break;
	}
#endif
#if LPCOM0_OPEN
	if(TASK_IDLE == uart_type[LPCOM0].uart_status)
	{
		Set100msTimer(TIMER_100MS_LPUART0_BUSY,UART_BUSY_OVERTIME);
	}
	else if(0 == Check100msTimer(TIMER_100MS_LPUART0_BUSY))
	{
		uart_type[LPCOM0].uart_status = TASK_IDLE;//ʱǿƸλ
		Reset_RstPeripheral0(ResetMskLpUart0);
		uart_type[LPCOM0].finish_send_flag = FALSE;				
		UartInit(IR_UART_COM, BAUD2400, DATA_BIT_8, STOPBIT_1, UART_EVEN_PARITY, UART_LPUART_Disable);
		uart_type[LPCOM0].send_state = UART_SEND_OVERTIME_S0;		
	}
	switch (uart_type[LPCOM0].send_state) 
	{
		case (UART_SEND_OVERTIME_S0): 
		{
			if (TRUE == uart_type[LPCOM0].start_send_flag) 
			{				
				uart_type[LPCOM0].start_send_flag = FALSE;
				Set100msTimer(TIMER_100MS_LPUART0_SEND, uart_type[LPCOM0].send_over_time);
				uart_type[LPCOM0].send_state = UART_SEND_OVERTIME_S1;
		    }
		} 
		break;
		case (UART_SEND_OVERTIME_S1):
		{
			if (TRUE == uart_type[LPCOM0].finish_send_flag) 
			{
				uart_type[LPCOM0].finish_send_flag = FALSE;
				uart_type[LPCOM0].send_state = UART_SEND_OVERTIME_S0;
			} 
			else if (0 == Check100msTimer(TIMER_100MS_LPUART0_SEND)) 
			{
				uart_type[LPCOM0].uart_status = TASK_IDLE;//ͳʱǿƸλ
				Reset_RstPeripheral0(ResetMskLpUart0);
				uart_type[LPCOM0].finish_send_flag = FALSE;
				UartInit(IR_UART_COM, BAUD2400, DATA_BIT_8, STOPBIT_1, UART_EVEN_PARITY, UART_LPUART_Disable);
				uart_type[LPCOM0].send_state = UART_SEND_OVERTIME_S0;
			} 
		} 
		break;
		default: 
		{
			uart_type[LPCOM0].send_state = UART_SEND_OVERTIME_S0;
		} 
		break;
	}
#endif
#if LPCOM1_OPEN
	if(TASK_IDLE == uart_type[LPCOM1].uart_status)
	{
		Set100msTimer(TIMER_100MS_LPUART1_BUSY,UART_BUSY_OVERTIME);
	}
	else if(0 == Check100msTimer(TIMER_100MS_LPUART1_BUSY))
	{
		uart_type[LPCOM1].uart_status = TASK_IDLE;//ʱǿƸλ
		Reset_RstPeripheral0(ResetMskLpUart1);
		uart_type[LPCOM1].finish_send_flag = FALSE;				
		#if MODULE_BLE_USED
			UartInit(BLE_UART_COM, BAUD38400, DATA_BIT_8, STOPBIT_1, UART_EVEN_PARITY,UART_LPUART_Disable);
		#else //NB
			UartInit(NB_UART_COM, BAUD9600, DATA_BIT_8, STOPBIT_1, UART_NONE_PARITY,UART_LPUART_Disable);
		#endif
		uart_type[LPCOM1].send_state = UART_SEND_OVERTIME_S0;		
	}
	switch (uart_type[LPCOM1].send_state) 
	{
		case (UART_SEND_OVERTIME_S0): 
		{
			if (TRUE == uart_type[LPCOM1].start_send_flag) 
			{				
				uart_type[LPCOM1].start_send_flag = FALSE;
				Set100msTimer(TIMER_100MS_LPUART1_SEND, uart_type[LPCOM1].send_over_time);
				uart_type[LPCOM1].send_state = UART_SEND_OVERTIME_S1;
		    }
		} 
		break;
		case (UART_SEND_OVERTIME_S1):
		{
			if (TRUE == uart_type[LPCOM1].finish_send_flag) 
			{
				uart_type[LPCOM1].finish_send_flag = FALSE;
				uart_type[LPCOM1].send_state = UART_SEND_OVERTIME_S0;
			} 
			else if (0 == Check100msTimer(TIMER_100MS_LPUART1_SEND)) 
			{
				uart_type[LPCOM1].uart_status = TASK_IDLE;//ͳʱǿƸλ
				Reset_RstPeripheral0(ResetMskLpUart1);
				uart_type[LPCOM1].finish_send_flag = FALSE;
				#if MODULE_BLE_USED
					UartInit(BLE_UART_COM, BAUD38400, DATA_BIT_8, STOPBIT_1, UART_EVEN_PARITY,UART_LPUART_Disable);
				#else //NB
					UartInit(NB_UART_COM, BAUD9600, DATA_BIT_8, STOPBIT_1, UART_NONE_PARITY,UART_LPUART_Disable);
				#endif
				uart_type[LPCOM1].send_state = UART_SEND_OVERTIME_S0;
			}
		}
		break;
		default: 
		{
			uart_type[LPCOM1].send_state = UART_SEND_OVERTIME_S0;
		} 
		break;
	}
#endif
}
//дڷͿ
u8 UartSendMachineIfIdle(u8 com)
{
	switch(com)
	{
#if COM0_OPEN
		case(COM0):
		{
			if(UART_RCV_OVERTIME_S0 == uart_type[COM0].send_state)
			{
				return TASK_IDLE;
			}
			else
			{
				return TASK_BUSY;
			}
		}
#endif
#if COM1_OPEN
		case(COM1):
		{
			if(UART_RCV_OVERTIME_S0 == uart_type[COM1].send_state)
			{
				return TASK_IDLE;
			}
			else
			{
				return TASK_BUSY;
			}
		}
#endif
#if COM2_OPEN
		case(COM2):
		{
			if(UART_RCV_OVERTIME_S0 == uart_type[COM2].send_state)
			{
				return TASK_IDLE;
			}
			else
			{
				return TASK_BUSY;
			}
		}
#endif		
#if COM3_OPEN
		case(COM3):
		{
			if(UART_RCV_OVERTIME_S0 == uart_type[COM3].send_state)
			{
				return TASK_IDLE;
			}
			else
			{
				return TASK_BUSY;
			}
		}
#endif			
#if LPCOM0_OPEN
		case(LPCOM0):
		{
			if(UART_RCV_OVERTIME_S0 == uart_type[LPCOM0].send_state)
			{
				return TASK_IDLE;
			}
			else
			{
				return TASK_BUSY;
			}
		}
#endif		

#if LPCOM1_OPEN
		case(LPCOM1):
		{
			if(UART_RCV_OVERTIME_S0 == uart_type[LPCOM1].send_state)
			{
				return TASK_IDLE;
			}
			else
			{
				return TASK_BUSY;
			}
		}
#endif		
		default:
		break;
	}
	return TASK_IDLE;
}
/*************************************************
Function:void UartStartReceiveIT(u8 com)
Description: ڽж
Input:  com:˿ں
Return: void
Others: ڲӿ
*************************************************/
static void UartStartReceiveIT(u8 com) 
{
	if ((LPCOM0 == com) || (LPCOM1 == com))
	{
		//uart_type[com].LP_Instance->SCON_f.TCIE = 0;  //رշж
		LPUart_EnableIrq(uart_type[com].LP_Instance,LPUartRxIrq);      ///<ж
		LPUart_EnableIrq(uart_type[com].LP_Instance,LPUartPEIrq);    //żУж	
		LPUart_EnableIrq(uart_type[com].LP_Instance,LPUartFEIrq);   //֡ж
	} 
	else 
	{
		//uart_type[com].Instance->SCON_f.TCIE = 0;  //رշж
		Uart_EnableIrq(uart_type[com].Instance,UartRxIrq);      ///<ж
		Uart_EnableIrq(uart_type[com].Instance,UartPEIrq);    //żУж	
		Uart_EnableIrq(uart_type[com].Instance,UartFEIrq);   //֡ж
	}
}
/*************************************************
Function:void UartStopReceiveIT(u8 com)
Description: ֹͣڽж
Input:  com:˿ں
Return: void
Others: ڲӿ
*************************************************/
static void UartStopReceiveIT(u8 com) 
{
	if ((LPCOM0 == com) || (LPCOM1 == com)) 
	{
		LPUart_DisableIrq(uart_type[com].LP_Instance,LPUartRxIrq);   //<ж
		LPUart_DisableIrq(uart_type[com].LP_Instance,LPUartPEIrq);   //żУж	
		LPUart_DisableIrq(uart_type[com].LP_Instance,LPUartFEIrq);   //֡ж
	} 
	else 
	{
		Uart_DisableIrq(uart_type[com].Instance,UartRxIrq);   ///<ж
		Uart_DisableIrq(uart_type[com].Instance,UartPEIrq);   //żУж	
		Uart_DisableIrq(uart_type[com].Instance,UartFEIrq);   //֡ж
	}
}

/*************************************************
Function:void UartPortCfg(u8 com)
Description: IOжϿʼ
Input:  com,˿ں
Return: void
Others: ڲӿ
*************************************************/
static void UartPortCfg(u8 com) 
{
    stc_gpio_cfg_t stcGpioCfg;
    DDL_ZERO_STRUCT(stcGpioCfg);

	stcGpioCfg.bOutputVal = TRUE;
#if COM0_OPEN
	if (com == COM0) 
	{
		Sysctrl_SetPeripheralGate(SysctrlPeripheralGpio,TRUE);  ///<ʹGPIOʱ
		// IO  
		///< TX
		stcGpioCfg.enDir = GpioDirOut;
		Gpio_Init(UART0_TX_PORT, UART0_TX_PIN, &stcGpioCfg);
		Gpio_SetAfMode(UART0_TX_PORT, UART0_TX_PIN, UART0_TX_AF);  //˿ΪURART0_TX

		///< RX
		stcGpioCfg.enDir = GpioDirIn;
		Gpio_Init(UART0_RX_PORT, UART0_RX_PIN, &stcGpioCfg);
		Gpio_SetAfMode(UART0_RX_PORT, UART0_RX_PIN, UART0_RX_AF);  //˿ΪURART0_RX
		Sysctrl_SetPeripheralGate(SysctrlPeripheralUart0,TRUE);  
		EnableNvic(UART0_2_IRQn, IrqLevel3, TRUE);  ///<ϵͳжʹ
	}
#endif

#if COM1_OPEN
	if (com == COM1) 
	{
		Sysctrl_SetPeripheralGate(SysctrlPeripheralGpio,TRUE);  ///<ʹGPIOʱ
		// IO  
		///< TX
		stcGpioCfg.enDir = GpioDirOut;
		Gpio_Init(UART1_TX_PORT, UART1_TX_PIN, &stcGpioCfg);
		Gpio_SetAfMode(UART1_TX_PORT, UART1_TX_PIN, UART1_TX_AF);  //˿ΪURART1_TX

		///< RX
		stcGpioCfg.enDir = GpioDirIn;
		Gpio_Init(UART1_RX_PORT, UART1_RX_PIN, &stcGpioCfg);
		Gpio_SetAfMode(UART1_RX_PORT, UART1_RX_PIN, UART1_RX_AF);  //˿ΪURART1_RX
		Sysctrl_SetPeripheralGate(SysctrlPeripheralUart1,TRUE);  
		EnableNvic(UART1_3_IRQn, IrqLevel3, TRUE);  ///<ϵͳжʹ
	}
#endif

#if COM2_OPEN
	if (com == COM2) 
	{
		Sysctrl_SetPeripheralGate(SysctrlPeripheralGpio,TRUE);  ///<ʹGPIOʱ
		// IO  
		///< TX
		stcGpioCfg.enDir = GpioDirOut;
		Gpio_Init(UART2_TX_PORT, UART2_TX_PIN, &stcGpioCfg);
		Gpio_SetAfMode(UART2_TX_PORT, UART2_TX_PIN, UART2_TX_AF);  //˿ΪURART2_TX

		///< RX
		stcGpioCfg.enDir = GpioDirIn;
		Gpio_Init(UART2_RX_PORT, UART2_RX_PIN, &stcGpioCfg);
		Gpio_SetAfMode(UART2_RX_PORT, UART2_RX_PIN, UART2_RX_AF);  //˿ΪURART2_RX
		Sysctrl_SetPeripheralGate(SysctrlPeripheralUart2,TRUE);  
		EnableNvic(UART0_2_IRQn, IrqLevel3, TRUE);  ///<ϵͳжʹ
	}
#endif

#if COM3_OPEN
	if (com == COM3) 
	{
		Sysctrl_SetPeripheralGate(SysctrlPeripheralGpio,TRUE);  ///<ʹGPIOʱ
		// IO  
		///< TX
		stcGpioCfg.enDir = GpioDirOut;
		Gpio_Init(UART3_TX_PORT, UART3_TX_PIN, &stcGpioCfg);
		Gpio_SetAfMode(UART3_TX_PORT, UART3_TX_PIN, UART3_TX_AF);  //˿ΪURART3_TX

		///< RX
		stcGpioCfg.enDir = GpioDirIn;
		Gpio_Init(UART3_RX_PORT, UART3_RX_PIN, &stcGpioCfg);
		Gpio_SetAfMode(UART3_RX_PORT, UART3_RX_PIN, UART3_RX_AF);  //˿ΪURART3_RX
		Sysctrl_SetPeripheralGate(SysctrlPeripheralUart3,TRUE);  
		EnableNvic(UART1_3_IRQn, IrqLevel3, TRUE);  ///<ϵͳжʹ
	}
#endif

#if LPCOM0_OPEN
	if (com == LPCOM0) 
	{
		Sysctrl_SetPeripheralGate(SysctrlPeripheralGpio,TRUE);  ///<ʹGPIOʱ
		// IO  
		///< TX
		stcGpioCfg.enDir = GpioDirOut;
		Gpio_Init(LPUART0_TX_PORT, LPUART0_TX_PIN, &stcGpioCfg);
		Gpio_SetAfMode(LPUART0_TX_PORT, LPUART0_TX_PIN,LPUART0_TX_AF);  //˿ΪLPURART0_TX

		///< RX
		stcGpioCfg.enPu = GpioPuEnable;  ///< ˿->
		stcGpioCfg.enDir = GpioDirIn;
		Gpio_Init(LPUART0_RX_PORT, LPUART0_RX_PIN, &stcGpioCfg);
		Gpio_SetAfMode(LPUART0_RX_PORT, LPUART0_RX_PIN,LPUART0_RX_AF);  //˿ΪLPURART0_RX
		Sysctrl_SetPeripheralGate(SysctrlPeripheralLpUart0,TRUE);  
		EnableNvic(LPUART0_IRQn, IrqLevel3, TRUE);  ///<ϵͳжʹ
	}
#endif

#if LPCOM1_OPEN
	if (com == LPCOM1) 
	{
		Sysctrl_SetPeripheralGate(SysctrlPeripheralGpio, TRUE);  ///<ʹGPIOʱ
		// IO  
		///< TX
		stcGpioCfg.enDir = GpioDirOut;
		Gpio_Init(LPUART1_TX_PORT, LPUART1_TX_PIN, &stcGpioCfg);
		Gpio_SetAfMode(LPUART1_TX_PORT, LPUART1_TX_PIN, LPUART1_TX_AF);  //˿ΪLPURART1_TX

		///< RX
		stcGpioCfg.enPu = GpioPuEnable;  ///< ˿->
		stcGpioCfg.enDir = GpioDirIn;
		Gpio_Init(LPUART1_RX_PORT, LPUART1_RX_PIN, &stcGpioCfg);
		Gpio_SetAfMode(LPUART1_RX_PORT, LPUART1_RX_PIN, LPUART1_RX_AF);  //˿ΪLPURART1_RX
		Sysctrl_SetPeripheralGate(SysctrlPeripheralLpUart1,TRUE);  
		EnableNvic(LPUART1_IRQn, IrqLevel3, TRUE);  ///<ϵͳжʹ		 
	}
#endif
}

/*************************************************
Function: u8 HAL_GetUart_Hander(u8 com,M0P_UART_TypeDef* UARTx)
Description: ͨںŻȡڵľ
Input:  comں
Return: void
Others: ڲӿ
*************************************************/
M0P_UART_TypeDef *HAL_GetUart_Hander(u8 com) 
{   
	M0P_UART_TypeDef *Addr;
	switch (com) 
	{
		case COM0:
			Addr = M0P_UART0;
		break;
		
		case COM1:
		    Addr = M0P_UART1;
		break;
		
		case COM2:
		    Addr = M0P_UART2;
		break;
		
		case COM3:
		    Addr = M0P_UART3;
		break;
    
		default:
		break;
	}
	return ((M0P_UART_TypeDef *)Addr);
}

M0P_LPUART_TypeDef *HAL_GetLPUart_Hander(u8 com) 
{
    M0P_LPUART_TypeDef *Addr;

	if (com == LPCOM0) 
	{
		Addr = M0P_LPUART0;
	}
	else
	{
	    Addr = M0P_LPUART1;
	}
    return ((M0P_LPUART_TypeDef *)Addr);
}
/*************************************************
Function: void HAL_UartCfg(u8 com)
Description: ڳʼ
Input:  comں
Return: void
Others: ڲӿ
*************************************************/
static void HAL_UartCfg(u8 com) 
{
	stc_uart_cfg_t stcCfg;
	stc_lpuart_cfg_t stclpCfg;
	M0P_LPUART_TypeDef *LPUARTx = NULL;
	M0P_UART_TypeDef *UARTx = NULL;
	//ȡھ
	if ((LPCOM0 == com) || (LPCOM1 == com)) 
	{
		DDL_ZERO_STRUCT(stclpCfg);
		LPUARTx = HAL_GetLPUart_Hander(com);		
		///< LPUART ʼ
		stclpCfg.enStopBit = (en_lpuart_stop_t)uart_type[com].para.StopBits;  ///< ֹͣλ
		stclpCfg.enMmdorCk = (en_lpuart_mmdorck_t)uart_type[com].para.Parity;    ///<У

		if (uart_type[com].para.LPModeFunction == UART_LPUART_Disable)  //ǵ͹ģʽ
		{
		    stclpCfg.stcBaud.enSclkSel = LPUartMskPclk;        ///<ʱԴ
		    stclpCfg.stcBaud.u32Sclk = Sysctrl_GetPClkFreq();  ///< PCLKȡ
		    stclpCfg.stcBaud.enSclkDiv = (en_lpuart_clkdiv_t)uart_type[com].para.OverSampling;    ///<Ƶ
		    stclpCfg.stcBaud.u32Baud = uart_type[com].para.BaudRate;  ///<
		} 
		else 
		{
		    stclpCfg.stcBaud.enSclkSel = LPUartMskRcl;  ///<ʱԴ
		    stclpCfg.stcBaud.u32Sclk = 38400;           ///< RCLʱƵ 38400Hz
		    stclpCfg.stcBaud.enSclkDiv = LPUartMsk4Or8Div;            ///<Ƶ
		    stclpCfg.stcBaud.u32Baud = uart_type[com].para.BaudRate;  ///<
		}
		stclpCfg.enRunMode = (en_lpuart_mode_t)uart_type[com].para.UARTMode;  ///<ģʽ
		LPUart_Init(LPUARTx, &stclpCfg);
//		LPUARTx->SADDR = 0;
//		LPUARTx->SADEN = 0;
//		///< LPUART ж
		LPUart_ClrStatus(LPUARTx, LPUartRC);      ///<ж
		LPUart_ClrStatus(LPUARTx, LPUartTC);      ///<巢ж
		LPUart_EnableIrq(LPUARTx, LPUartRxIrq);   ///<ʹܽж
		LPUart_DisableIrq(LPUARTx, LPUartTxIrq);  ///<ֹж
	} 
	else
	{
		DDL_ZERO_STRUCT(stcCfg);
		UARTx = HAL_GetUart_Hander(com);
		///< UART Init		
		stcCfg.enStopBit = (en_uart_stop_t)uart_type[com].para.StopBits;        ///<ֹͣλ
		stcCfg.enMmdorCk = (en_uart_mmdorck_t)uart_type[com].para.Parity;          ///<ż	
		stcCfg.stcBaud.u32Pclk = Sysctrl_GetPClkFreq();  ///<ʱӣPCLKƵֵ	
		stcCfg.stcBaud.enClkDiv = (en_uart_clkdiv_t)uart_type[com].para.OverSampling;  ///<ͨƵ
		stcCfg.stcBaud.u32Baud = uart_type[com].para.BaudRate;  ///<
		
		stcCfg.enRunMode = (en_uart_mode_t)uart_type[com].para.UARTMode;        ///<ģʽ
		Uart_Init(UARTx, &stcCfg);  ///<ڳʼ
//		UARTx->SADDR = 0;
//		UARTx->SADEN = 0;
//		///< UARTжʹ
		Uart_ClrStatus(UARTx, UartRC);     ///<
		Uart_ClrStatus(UARTx, UartTC);     ///<
		Uart_EnableIrq(UARTx, UartRxIrq);  ///<ʹܴڽж
		Uart_DisableIrq(UARTx, UartTxIrq);  ///<ֹڷж
	}
}
/*************************************************
Function:void UartTransmitIT(u8 com)
Description: ڷ
Input:  com˿ںţ
return: void
Others: ڲӿ
*************************************************/
static void UartTransmitIT(u8 com) 
{
    if (uart_type[com].send_len > 0) 
	{
		if ((LPCOM0 == com) || (LPCOM1 == com))  
		{
			LPUart_SendDataIt(uart_type[com].LP_Instance, *uart_type[com].send_point); 		
		} 
		else 
		{
		    Uart_SendDataIt(uart_type[com].Instance, *uart_type[com].send_point); 		  //
		}
		uart_type[com].send_point++;
		uart_type[com].send_len--;
	}

    else if (0 == uart_type[com].send_len) 
	{
		if(LPCOM0 == com)//    
		{
			LPUart_DisableIrq(uart_type[com].LP_Instance,LPUartTxIrq);///<ֹж
			LPUart_EnableIrq(uart_type[com].LP_Instance,LPUartRxIrq); ///<ⷢɣж
            ir_tx_end_rdy_flag = TRUE;
			Set100msTimer(TIMER_100MS_IR_TX_38K_END,10); 

		} 
		else if(LPCOM1 == com)//NB
		{
			LPUart_DisableIrq(uart_type[com].LP_Instance,LPUartTxIrq);///<ֹж
		}
		else 
		{
		    Uart_DisableIrq(uart_type[com].Instance,UartTxIrq);///<ֹж
		}		
		*uart_type[com].send_status_point = TASK_IDLE;
		uart_type[com].uart_status = TASK_IDLE;		
		uart_type[com].finish_send_flag = TRUE;
    }
}

/*************************************************
Function:void UartReceiveIT(u8 com)
Description: ڽж
Input:  com:˿ں
Return: void
Others: ڲӿ
*************************************************/
static void UartReceiveIT(u8 com)
{
    volatile u8 temp = 0;  	
	if (uart_type[com].rcv_len < UP_BUF_SIZE) 
	{
		uart_type[com].rcv_flag = 1;  //յ
		switch(com)
		{
#if COM0_OPEN
			case(COM0):
			{
				s_uart0_rcv_buf[uart_type[com].rcv_len] = Uart_ReceiveData(M0P_UART0);///ȡ
			}
			break;
#endif		
			
#if COM1_OPEN
			case(COM1):
			{
				NBUartReceiveIT();					//NB uartӿڣ
				uart_type[com].rcv_len = 0;
				//s_uart1_rcv_buf[uart_type[com].rcv_len] = Uart_ReceiveData(M0P_UART1);///ȡ
			}
			break;
#endif
			
#if COM2_OPEN
			case(COM2):
			{
				s_uart2_rcv_buf[uart_type[com].rcv_len] = Uart_ReceiveData(M0P_UART2);///ȡ
			}
			break;
#endif

#if COM3_OPEN
			case(COM3):
			{
				s_uart3_rcv_buf[uart_type[com].rcv_len] = Uart_ReceiveData(M0P_UART3);///ȡ
			}
			break;
#endif

#if LPCOM0_OPEN
			case (LPCOM0):
			{
				s_lpuart0_rcv_buf[uart_type[com].rcv_len] = LPUart_ReceiveData(M0P_LPUART0);///ȡ
			}
			break;
#endif		

#if LPCOM1_OPEN
			case (LPCOM1):
			{
			#if MODULE_BLE_USED
				uart_type[com].rcv_flag = 0;  
				BleUartReceiveIT();			  
				uart_type[com].rcv_len = 0;
			#else
				uart_type[com].rcv_flag = 0;  //NB֡䳬ʱж
				NBUartReceiveIT();			  //NB uartӿڣ
				uart_type[com].rcv_len = 0;
			#endif
			}
			break;
#endif			
		}
		uart_type[com].rcv_len++;
	} 
	else 
	{
		if ((LPCOM0 == com) || (LPCOM1 == com))    
		{
			temp = LPUart_ReceiveData(uart_type[com].LP_Instance);			
		}
		else
		{
			temp = Uart_ReceiveData(uart_type[com].Instance);			
		}
		uart_type[com].rcv_len = 0;
	}  
}
/*************************************************
Function:void UartIRQHandler(u8 com)
Description: жϴ
Input:  com:˿ں
Return: None
Others: жϳռ䣻ڲӿ
*************************************************/
static void UartIRQHandler(u8 com) 
{
	if((LPCOM0 == com) || (LPCOM1 == com))   //LPUART
	{
		//żУж
		if (TRUE == LPUart_GetStatus(uart_type[com].LP_Instance,LPUartPE)) 
		{
		    LPUart_ClrStatus(uart_type[com].LP_Instance,LPUartPE); 
		}
		//֡ж
		if (TRUE == LPUart_GetStatus(uart_type[com].LP_Instance,LPUartFE)) 
		{		    
			LPUart_ClrStatus(uart_type[com].LP_Instance,LPUartFE); 
		}
		//ж
		if (TRUE == LPUart_GetStatus(uart_type[com].LP_Instance, LPUartTC)) 
		{
			LPUart_ClrStatus(uart_type[com].LP_Instance,LPUartTC); 
		    UartTransmitIT(com);
		}
		//ж յݣҽжϿ
		if ((TRUE == LPUart_GetStatus(uart_type[com].LP_Instance, LPUartRC)) && (TRUE == uart_type[com].LP_Instance->SCON_f.RCIE)) 
		{
			LPUart_ClrStatus(uart_type[com].LP_Instance,LPUartRC); 
		    UartReceiveIT(com);			
		}
	}
	else 
	{	
		//żУж
		if (TRUE == Uart_GetStatus(uart_type[com].Instance,UartPE)) 
		{
		    Uart_ClrStatus(uart_type[com].Instance,UartPE); 
		}
		//֡ж
		if (TRUE == Uart_GetStatus(uart_type[com].Instance,UartFE)) 
		{		    
			Uart_ClrStatus(uart_type[com].Instance,UartFE); 
		}
		//ж
		if (TRUE == Uart_GetStatus(uart_type[com].Instance, UartTC)) 
		{
			Uart_ClrStatus(uart_type[com].Instance,UartTC); 
		    UartTransmitIT(com);			
		}
		//ж
		if ((TRUE == Uart_GetStatus(uart_type[com].Instance, UartRC)) && (TRUE == uart_type[com].Instance->SCON_f.RCIE))  
		{
			Uart_ClrStatus(uart_type[com].Instance,UartRC); 
		    UartReceiveIT(com);			
		}		
	}
}





